const P = Promise;

/**
 * This function periodically invokes a synchronous function (a "truth test")
 * until it returns truthy value. The return value is a promise that settles
 * to these possible outcomes:
 *  - resolve if the truth test passes
 *  - reject if the truth test raises an Error
 *  - reject if the the operation times out
 * 
 * @async Promise
 * @param {Function} test - a function that returns a boolean
 * @param {Object} options
 * @param {Number} options.ttl - timeout in millis, default 10000
 * @param {Number} options.frequency - polling frequency in millis, default 100
 * @return void
 */
const until = (test, options={}) => {
  let defaults = {ttl: 10000, frequency: 100};
  let opts = {...defaults, ...options};

  return new P((resolve, reject) => {
    let expire = () => reject(new Error('polling operation timed out'));
    let timeout = setTimeout(expire, opts.ttl);

    let done = (e) => {
      clearTimeout(timeout);
      e ? reject(e) : resolve();
    };

    let poll = () => {
      try {
        if (test()) {
          return done();
        }
      }
      catch (e) {
        return done(e);
      }

      setTimeout(poll, opts.frequency);
    };

    poll();
  });
};

export default {until};
